//
//SJJ_ADC_Driver
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//
//

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.IO.Ports;
using Microsoft.SPOT.Hardware.SJJ;

namespace Microsoft.SPOT.Hardware.SJJ.SJJ_ADC_Driver
{

    //-----------------------------------------------------------------------------------------------
    // ADC Command structure                                                                        |
    //-----------------------------------------------------------------------------------------------
    // | Byte 0 |   Byte 1    |   Byte 2    |    Byte 3    | Byte 4 ...byte n...                    |
    // ----------------------------------------------------------------------------------------------
    // | Start  |   Command   | ADC Channel |  Data Length | Data HiByte | Data | ... | Data LoByte |
    // ----------------------------------------------------------------------------------------------
    // |  0x02  | ADC_Command | 0 only      | # Data Bytes | Data bytes...                          |
    // ----------------------------------------------------------------------------------------------
    public class SJJ_ADC_Driver
    {
        //ADC Definitions
        protected const int ADC_SOF_IDX = 0;   // START-OF-FRAME INDEX
        protected const int ADC_COMMAND_BYTE_IDX = 1;
        protected const int ADC_CHANNEL_BYTE_IDX = 2;
        protected const int ADC_DATA_LENGTH_IDX = 3;
        protected const int ADC_DATA_START_IDX = 4;

        protected const int ADC_CONVERSION_BYTES = 2;   // 16-bit ADC
        protected const int MAX_ADC_CMD_LEN = (4 + ADC_CONVERSION_BYTES);

        public const int ADC_CONVERSION_ERROR = 0x7000;  // Out of range value to indicate conversion failure
        public const int ADC_COM_ERROR = 0x7700; // Null COM port handle
        public const int ADC_NEG_RANGE = 0x9E58;  // ADC count range is +/- 25,000 counts
        public const int ADC_POS_RANGE = 0x61A8;

        protected SerialPort myADC;
//->        protected static SerialPort.Configuration myADCConfig;

        // Command enumerations
        protected enum ADC_Command
        {
            eInvalid,
            eSetReadReg,
            eSTX,
            eWriteADCSource,
            eADCOn,
            eADCOff,
            eSetADCClkDiv4,
            eSetADCClkDiv16,
            eSampleADC,
            eSYN = 22
        }

        public enum ADC_ClkDiv
        {
            eADCDivide4, 
            eADCDivide16
        }

        public enum ADC_Source
        {
            eAnalogChannel0,
            eAnalogChannel1,
            eAnalogChannel2,
            eAnalogChannel3,
            eAnalogChannel4,
            eBatteryVoltage
        }

        // Read enumerations
        protected enum ADC_Read_Reg
        {
            eReadStatus,
            eReadEnd
        }

        // Status enumerations
        public enum ADC_Status
        {
            eStatusInit,
            eStatusWriteSuccess,
            eStatusReadSuccess,
            eStatusReadFail,
            eStatusSynchError,
            eStatusCommandLengthError,
            eStatusSynchSuccess,
            eStatusSetReadSuccess,
            eStatusSetReadFail,
            eStatusADCSourceError,
            eStatusADCComHandleError
        }

        // ADC Command arrays
        protected byte[] bTurnADCOn = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eADCOn, 0, 0 };
        protected byte[] bTurnADCOff = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eADCOff, 0, 0 };
        protected byte[] bReadStatus = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eSetReadReg, 0, 1, (byte) ADC_Read_Reg.eReadStatus };

        // Replace last byte with channel number
        protected byte[] bSetADC2Ch = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eWriteADCSource, 0, 1, 0x00 };

        protected byte[] bSampleADC = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eSampleADC, 0, 0 };
        protected byte[] bSetADCDivide4 = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eSetADCClkDiv4, 0, 0 };
        protected byte[] bSetADCDivide16 = new byte[] { (byte)ADC_Command.eSTX, (byte)ADC_Command.eSetADCClkDiv16, 0, 0 };

        public SerialPort ADC_Open()
        {
            // Baud rate does not matter for virtual COM ports
//->            myADCConfig = new SerialPort.Configuration(SerialPort.Serial.COM4, SerialPort.BaudRate.Baud115200, false);
//->            myADC = new SerialPort(myADCConfig);
            myADC = new SerialPort(SerialPorts.COM4);
            myADC.Handshake = Handshake.None;

            // Open ADC virtual serial port
            myADC.Open();

            return (myADC);
        }

        public ADC_Status ADC_On(SerialPort myAdc)
        {
            ADC_Status eReturnVal = ADC_Status.eStatusADCComHandleError;

            if (myAdc != null)
            {
                byte[] bReadBuf = new byte[1];

                myAdc.Write(bTurnADCOn, 0, bTurnADCOn.Length);

                // Read and return status
                myAdc.Write(bReadStatus, 0, bReadStatus.Length);
//->                myAdc.Read(bReadBuf, 0, 1, 1);
                myAdc.Read(bReadBuf, 0, 1);

                eReturnVal = (ADC_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public ADC_Status ADC_Off(SerialPort myAdc)
        {
            ADC_Status eReturnVal = ADC_Status.eStatusADCComHandleError;

            if (myAdc != null)
            {
                byte[] bReadBuf = new byte[1];

                myAdc.Write(bTurnADCOff, 0, bTurnADCOff.Length);

                // Read and return status
                myAdc.Write(bReadStatus, 0, bReadStatus.Length);
//->                myAdc.Read(bReadBuf, 0, 1, 1);
                myAdc.Read(bReadBuf, 0, 1);

                eReturnVal = (ADC_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public ADC_Status ADC_SetChannel(ADC_Source eAnalogChannel, SerialPort myAdc)
        {
            ADC_Status eReturnVal = ADC_Status.eStatusADCComHandleError;

            if (myAdc != null)
            {
                byte[] bReadBuf = new byte[1];

                // Set channel
                bSetADC2Ch[bSetADC2Ch.Length - 1] = (byte)eAnalogChannel;
                myAdc.Write(bSetADC2Ch, 0, bSetADC2Ch.Length);

                // Clear channel
                bSetADC2Ch[bSetADC2Ch.Length - 1] = 0x00;

                // Read and return status
                myAdc.Write(bReadStatus, 0, bReadStatus.Length);
//->                myAdc.Read(bReadBuf, 0, 1, 1);
                myAdc.Read(bReadBuf, 0, 1);

                eReturnVal = (ADC_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public int ADC_Read(SerialPort myAdc)
        {
            int iReturnVal = ADC_COM_ERROR;

            if (myAdc != null)
            {
                byte[] bReadBuf = new byte[2];

                myAdc.Write(bSampleADC, 0, bSampleADC.Length);
//->                myAdc.Read(bReadBuf, 0, bReadBuf.Length, 1);
                myAdc.Read(bReadBuf, 0, bReadBuf.Length);

                // Convert bytes to 16-bit integer
                iReturnVal = (int)bReadBuf[1];
                iReturnVal = (int)(iReturnVal << 8);
                iReturnVal |= (int)bReadBuf[0];
            }

            return (iReturnVal);
        }

        public ADC_Status ADC_SetClkDiv(ADC_ClkDiv eADCDivide, SerialPort myAdc)
        {
            ADC_Status eReturnVal = ADC_Status.eStatusADCComHandleError;

            if (myAdc != null)
            {
                byte[] bReadBuf = new byte[1];

                switch (eADCDivide)
                {
                    case ADC_ClkDiv.eADCDivide16:
                    {
                        myAdc.Write(bSetADCDivide16, 0, bSetADCDivide16.Length);
                        break;
                    }
                    case ADC_ClkDiv.eADCDivide4:
                    {
                        myAdc.Write(bSetADCDivide4, 0, bSetADCDivide4.Length);
                        break;
                    }
                    default:
                    {
                        myAdc.Write(bSetADCDivide4, 0, bSetADCDivide4.Length);
                        break;
                    }
                }

                // Read and return status
                myAdc.Write(bReadStatus, 0, bReadStatus.Length);
//->                myAdc.Read(bReadBuf, 0, 1, 1);
                myAdc.Read(bReadBuf, 0, 1);

                eReturnVal = (ADC_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public ADC_Status ADC_ReadStatus(SerialPort myAdc)
        {
            ADC_Status eReturnVal = ADC_Status.eStatusADCComHandleError;
            byte[] bReadBuf = new byte[1];

            if (myAdc != null)
            {
                myAdc.Write(bReadStatus, 0, bReadStatus.Length);
//->                myAdc.Read(bReadBuf, 0, 1, 1);
                myAdc.Read(bReadBuf, 0, 1);

                eReturnVal = (ADC_Status) bReadBuf[0];
            }

            return (eReturnVal);
        }
    }
}
