﻿
//
//SPI_Serial_Drivers
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.IO.Ports;

namespace Microsoft.SPOT.Hardware.SJJ.SPI_Serial_Driver
{
    public static class Version
    {
        public const string SpiSerVersion = "SJJ SPI-Serial Driver V4.1.1.1";
    }

    public class SPI_Serial_Driver
    {
        //Default configurations
        protected BaudRate ssBaudRate = BaudRate.Baudrate115200;
        protected Parity ssParity = Parity.None;
        protected int ssDataBits = 8;
        protected StopBits ssStopBits = StopBits.One;
        protected ushort[] usSpiSerConfig = new ushort[1];
        protected ushort[] usSpiSerRTS = new ushort[1];
        protected ushort[] usWriteChar = new ushort[1];

        private const UInt16 uiMaximConfigRM = 0x0400;  // Enable receive interrupt
        private const UInt16 uiMaximWritePrefix = 0x8000;   // OR with 8-bit data on lower byte
        private const UInt16 uiMaximTxDisable = 0x0400;  //Disable char transmit on data write
        private const UInt16 uiMaximSetRTS = 0x0000;  //Halt incoming data: ~RTS = 0 -> RTS= 1; update RTS only no char write
        private const UInt16 uiMaximClearRTS = 0x0200;  //Enable incoming data: ~RTS = 1 -> RTS= 0; update RTS only no char write
        private const ushort uiMaximCTSMask = 0x0200;   // When CTS set, tranmits can be done
        private const UInt16 uiMaximReadDataMask = 0x00FF;   // AND with data read for data out
        private const UInt16 uiMaximReadCommand = 0x0000;
        private const UInt16 uiMaximReadDataValid = 0x8000;   // AND with DOUT and > 0 if data valid
        private const UInt16 uiMaximTxFull = 0x4000;   // AND with DOUT and if 0 Tx full

        public ushort[] uiReadCommand = new ushort[] { uiMaximReadCommand };


        //Constructor
        public SPI_Serial_Driver()
        {
            usSpiSerConfig[0] = 0xC001; //115200, 8, N, 1
        }

        //Overloaded Configuration method
        public ushort[] SPI_Serial_Config()
        {
            //Enable receive interrupt
            usSpiSerConfig[0] |= uiMaximConfigRM;

            // Return constructor default configuration
            return (usSpiSerConfig);
        }

        public ushort[] SPI_Serial_Config(BaudRate setBaudRate)
        {
            SetBaudConfig(setBaudRate);

            //Enable receive interrupt
            usSpiSerConfig[0] |= uiMaximConfigRM;

            return (usSpiSerConfig);
        }


        public ushort[] SPI_Serial_Config(BaudRate setBaudRate, int setDataBits)
        {
            SetBaudConfig(setBaudRate);

            SetDataConfig(setDataBits);

            //Enable receive interrupt
            usSpiSerConfig[0] |= uiMaximConfigRM;

            return (usSpiSerConfig);
        }

        public ushort[] SPI_Serial_Config(BaudRate setBaudRate, int setDataBits, StopBits setStopBits)
        {
            SetBaudConfig(setBaudRate);

            SetDataConfig(setDataBits);

            switch (setStopBits)
            {
                case StopBits.One:
                    {
                        usSpiSerConfig[0] &= 0xFFBF;
                        break;
                    }
                case StopBits.Two:
                    {
                        usSpiSerConfig[0] |= 0x0040;
                        break;
                    }
                default:
                    {
                        //default to 1 stop bit
                        usSpiSerConfig[0] &= 0xFFBF;
                        break;
                    }
            }

            //Enable receive interrupt
            usSpiSerConfig[0] |= uiMaximConfigRM;

            return (usSpiSerConfig);
        }

        public ushort[] ClearRTS()
        {
            usSpiSerRTS[0] = uiMaximWritePrefix | uiMaximTxDisable | uiMaximClearRTS;

            return (usSpiSerRTS);
        }

        public bool CheckCTS(ushort uiReadData)
        {
            bool bReturnVal = false;

            if ((uiReadData & uiMaximCTSMask) != 0)
            {
                bReturnVal = true;
            }

            return (bReturnVal);
        }

        public ushort[] WriteChar(ushort usChar)
        {
            usWriteChar[0] = (ushort) (usChar & uiMaximReadDataMask);
            usWriteChar[0] |= uiMaximWritePrefix;

            return (usWriteChar);
        }

        public bool CheckDataValid(ushort uiReadData)
        {
            bool bReturnVal = false;

            if ((uiMaximReadDataValid & uiReadData) != 0)
            {
                bReturnVal = true;
            }

            return (bReturnVal);
        }

        public bool CheckTxEmpty(ushort uiReadData)
        {
            bool bReturnVal = false;

            if ((uiReadData & uiMaximTxFull) != 0)
            {
                bReturnVal = true;
            }

            return (bReturnVal);
        }

        private void SetBaudConfig(BaudRate setBaudRate)
        {
            // Clear baudrate bits
            usSpiSerConfig[0] &= 0xFFF0;

            switch (setBaudRate)
            {
                case BaudRate.Baudrate115200:
                    {
                        usSpiSerConfig[0] |= 0x0001;
                        break;
                    }
                case BaudRate.Baudrate57600:
                    {
                        usSpiSerConfig[0] |= 0x0002;
                        break;
                    }
                case BaudRate.Baudrate38400:
                    {
                        usSpiSerConfig[0] |= 0x0009;
                        break;
                    }
                case BaudRate.Baudrate19200:
                    {
                        usSpiSerConfig[0] |= 0x000A;
                        break;
                    }
                case BaudRate.Baudrate9600:
                    {
                        usSpiSerConfig[0] |= 0x000B;
                        break;
                    }
                case BaudRate.Baudrate4800:
                    {
                        usSpiSerConfig[0] |= 0x000C;
                        break;
                    }
                case BaudRate.Baudrate2400:
                    {
                        usSpiSerConfig[0] |= 0x000D;
                        break;
                    }
                case BaudRate.Baudrate1200:
                    {
                        usSpiSerConfig[0] |= 0x000E;
                        break;
                    }
                case BaudRate.Baudrate600:
                    {
                        usSpiSerConfig[0] |= 0x000F;
                        break;
                    }
                default:
                    {
                        //default to 115200 baud
                        usSpiSerConfig[0] |= 0x0001;
                        break;
                    }
            }
        }

        private void SetDataConfig(int setDataBits)
        {
            if (setDataBits == 8)
            {
                usSpiSerConfig[0] &= 0xFFEF;
            }
            else if (setDataBits == 7)
            {
                usSpiSerConfig[0] |= 0x0010;
            }
            else
            {
                //default to 8 data bits
                usSpiSerConfig[0] &= 0xFFEF;
            }
        }
    }

}
