//
//SJJ_PWM_Driver
//SJJ Embedded Micro Solutions
//www.sjjmicro.com / www.seanliming.com
//Copyright (c) 2006-2012 SJJ Embedded Micro Solutions    All Rights Reserved
//
//Source code is provided as is without any warranty. Any errors or omissions shall not imply any 
//liability for direct or indirect consequences arising from the use of this source code. The authors
//and reviewers make no warranty for the correctness or for the use of this source code, and asseme no
//liability for direct or indirect damages of any kind arising from the use of this source code.
//
//The author resrves the right to make changes in this source code without notice and without
//incurring any liability.
//

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.IO.Ports;

namespace Microsoft.SPOT.Hardware.SJJ.SJJ_PWM_Driver
{

    //-----------------------------------------------------------------------------------------------
    // Command structure                                                                            |
    //-----------------------------------------------------------------------------------------------
    // | Byte 0 |   Byte 1    |   Byte 2    |    Byte 3    | Byte 4 ...byte n...                    |
    // ----------------------------------------------------------------------------------------------
    // | Start  |   Command   | PWM Channel |  Data Length | Data HiByte | Data | ... | Data LoByte |
    // ----------------------------------------------------------------------------------------------
    // |  0x02  | PWM_Command | 0, 1, or 2  | # Data Bytes | Data bytes...                          |
    // ----------------------------------------------------------------------------------------------

    public class SJJ_PWM_Driver
    {
        // Command enumerations
        protected enum PWM_Command
        {
            eInvalid,
            eSetReadReg,
            eSTX,
            eWriteTermCnt,
            eWriteDutyCycle,
            eWriteEnable,
            eWriteInvert,
            eWriteGpwma,
            eWriteGpwmb,
            eWriteCPLDControl,
            eSYN = 22
        }

        protected enum PWM_Read
        {
            eReadTermCnt,
            eReadDutyCycle,
            eReadEnable,
            eReadInvert,
            eReadStatus,
            eNumRead
        }

        // Status enumerations
        public enum PWM_Status
        {
            eStatusInit,
            eStatusWriteSuccess,
            eStatusReadSuccess,
            eStatusReadFail,
            eStatusSynchError,
            eStatusCommandLengthError,
            eStatusSynchSuccess,
            eStatusSetReadSuccess,
            eStatusSetReadFail,
            eStatusOpenSuccess,
            eStatusOpenFail,
            eStatusCloseSuccess,
            eStatusCloseFail,
            eStatusPWMComHandleError,
            eNumStatus
        }

        public enum PWM_Channel
        {
            ePWM1,
            ePWM2_CPLD,
            ePWM3_CPLD
        }

        protected const int MAX_PWM_CMD_LEN = 8;
        protected const int MAX_PWM_READ_LEN = 2;
        protected const int PWM_SOF_IDX = 0;   // START-OF-FRAME INDEX
        protected const int PWM_COMMAND_BYTE_IDX = 1;
        protected const int PWM_CHANNEL_BYTE_IDX = 2;
        protected const int PWM_DATA_LENGTH_IDX = 3;
        protected const int PWM_DATA_START_IDX = 4;
        protected const UInt32 PWM_CLOCK_FREQ = 14745600;  // Clock frequency in Hz
        protected const UInt32 PWM_MAX_TERMINAL_COUNT = 0xFFFF; // Terminal count register is 16-bit
        protected const byte PLA_INVERT_mask = 0x20; // CPLD PWM Channel a invert control
        protected const byte PLB_INVERT_mask = 0x40; // CPLD PWM Channel b invert control
        protected const int CPLD_PWM_MAX_COUNT = 256; // CPLD count for iput freq divisor and duty cycle

        protected SerialPort myPWM;
//->        protected static SerialPort.Configuration myPWMConfig;
        protected static byte bCpldControlShadowReg;

        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // PWM1 Calculations
        // PWMxTermCnt   = (PWM_CLOCK_FREQ / <Target Freq in Hz>) - 1
        // PWMxDutyCycle = (<duty cycle as deciman> * (PWMxTermCnt + 1)) - 1
        //
        // To produce a PWM1 output of 500 kHz and 20% duty cycle:
        // 1. Calculate PWMxTermCnt = (PWM_CLOCK_FREQ / 500000)  1 = 28 (decimal) = 0x001C: HiByte = 0x00, LoByte = 0x1C
        // 2. Calculate PWMxDutyCycle = (0.2  (28 + 1))  1 = 5 (decimal) = 0x0005: HiByte = 0x00, LoByte = 0x05
        // 3. Calculate PWMxDutyCycle for 50% = (0.5 x (28 + 1)) -1 = 13 (decimal) = 0x000D: HiByte = 0x00, LoByte = 0x0D
        ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // PWM2 & PWM3 (CPLD) Calculations
        // Dutycycle = (8-bit count) / 256
        // For 10% dutycycle 8-bit count = 26 (decimal) = 0x1A
        // For 75% dutycycle 8-bit count = 192 (decimal) = 0xC0
        //////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        // Replace last 2 bytes with HiByte, LoByte of Terminal Count
        protected byte[] bSetFreq = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteTermCnt, 0, 2, 0x00, 0x00 };

        protected byte[] bTurnPWM1on = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteEnable, 0, 2, 0, 1 };
        protected byte[] bTurnPWM1off = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteEnable, 0, 2, 0, 0 };
        protected byte[] bSetPWM1InvertOn = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteInvert, 0, 2, 0, 1 };
        protected byte[] bSetPWM1InvertOff = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteInvert, 0, 2, 0, 0 };

        // Replace last byte with CPLD Control register byte to be written
        protected byte[] bSetCpldPwmControl = new byte[] { (byte)PWM_Command.eSTX, (byte) PWM_Command.eWriteCPLDControl, 0, 1, 0x00 };

        // Replace last 2 bytes with hi-byte/lo-byte of duty cycle register
        protected byte[] bSetPWM1DutyCycle = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteDutyCycle, 0, 2, 0x00, 0x00 };

        // Replace last byte with duty cycle register byte
        protected byte[] bSetPWM2DutyCycle = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteGpwma, 0, 1, 0x00 };
        protected byte[] bSetPWM3DutyCycle = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eWriteGpwmb, 0, 1, 0x00 };

        protected byte[] bSetStatusRead = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eSetReadReg, 0, 1, (byte)PWM_Read.eReadStatus };
        protected byte[] bSetReadTerminalCount = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eSetReadReg, 0, 1, (byte)PWM_Read.eReadTermCnt };
        protected byte[] bSetReadDutyCycle = new byte[] { (byte)PWM_Command.eSTX, (byte)PWM_Command.eSetReadReg, 0, 1, (byte)PWM_Read.eReadDutyCycle };


        public SerialPort PWM_Open()
        {
            // Baud rate does not matter for virtual COM ports
//->            myPWMConfig = new SerialPort.Configuration(SerialPort.Serial.COM3, SerialPort.BaudRate.Baud115200, false);
//->            myPWM = new SerialPort(myPWMConfig);
//->            myPWM = new SerialPort("COM3", 115200, Parity.None, 8, StopBits.One);
            myPWM = new SerialPort(SerialPorts.COM3);
            myPWM.Handshake = Handshake.None;

            if (myPWM != null)
            {
                // Open virtual serial port
                myPWM.Open();

                // Set CPLD Control Register to 0
                myPWM.Write(bSetCpldPwmControl, 0, bSetCpldPwmControl.Length);
                bCpldControlShadowReg = 0;
            }

            return (myPWM);
        }

        public PWM_Status PWM_On(SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;

            // Turn PWM1 on
            if (myPwm != null)
            {
                byte[] bReadBuf = new byte[1];

                myPwm.Write(bTurnPWM1on, 0, bTurnPWM1on.Length);

                // Read and return status
                myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                myPwm.Read(bReadBuf, 0, 1, 1);
                myPwm.Read(bReadBuf, 0, 1);

                eReturnVal = (PWM_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public PWM_Status PWM_Off(SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;

            // Turn PWM1 off
            if (myPwm != null)
            {
                byte[] bReadBuf = new byte[1];

                myPwm.Write(bTurnPWM1off, 0, bTurnPWM1off.Length);

                // Read and return status
                myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                myPwm.Read(bReadBuf, 0, 1, 1);
                myPwm.Read(bReadBuf, 0, 1);

                eReturnVal = (PWM_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public PWM_Status PWM_SetFreq(UInt32 uiPWMFreq, SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;
            UInt32 uiPWMxTermCnt;

            // Calculate terminal count
            uiPWMxTermCnt = (PWM_CLOCK_FREQ / uiPWMFreq) - 1;

            // Check that terminal count (and target frequency) is in range
            if (uiPWMxTermCnt <= PWM_MAX_TERMINAL_COUNT)
            {
                // Set terminal count in command string
                bSetFreq[bSetFreq.Length - 1] = (byte)(uiPWMxTermCnt & 0x000000FF);
                bSetFreq[bSetFreq.Length - 2] = (byte)((uiPWMxTermCnt >> 8) & 0x000000FF);

                // Write terminal count command to PWM virutal port
                if (myPwm != null)
                {
                    byte[] bReadBuf = new byte[1];
                    myPwm.Write(bSetFreq, 0, bSetFreq.Length);


                    // Read and return status
                    myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                    myPwm.Read(bReadBuf, 0, 1, 1);
                    myPwm.Read(bReadBuf, 0, 1);

                    eReturnVal = (PWM_Status)bReadBuf[0];
                }
            }

            return (eReturnVal);
        }

        public PWM_Status PWM_SetInvert(PWM_Channel ePWMChannel,  bool bInvert, SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;

            // Set invert state
            if (myPwm != null)
            {
                byte[] bReadBuf = new byte[1];

                switch (ePWMChannel)
                {
                    case PWM_Channel.ePWM1:
                    {
                        if (bInvert)
                        {
                            myPwm.Write(bSetPWM1InvertOn, 0, bSetPWM1InvertOn.Length);
                        }
                        else
                        {
                            myPwm.Write(bSetPWM1InvertOff, 0, bSetPWM1InvertOff.Length);
                        }

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    case PWM_Channel.ePWM2_CPLD:
                    {
                        if (bInvert)
                        {
                            bCpldControlShadowReg |= PLA_INVERT_mask;
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = bCpldControlShadowReg;
                            myPwm.Write(bSetCpldPwmControl, 0, bSetCpldPwmControl.Length);

                            // Clear control reg command array byte
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = 0;
                        }
                        else
                        {
                            byte bClearMask = PLA_INVERT_mask;
                            bCpldControlShadowReg &= (byte) ~bClearMask;
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = bCpldControlShadowReg;
                            myPwm.Write(bSetCpldPwmControl, 0, bSetCpldPwmControl.Length);

                            // Clear control reg command array byte
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = 0;
                        }

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    case PWM_Channel.ePWM3_CPLD:
                    {
                        if (bInvert)
                        {
                            bCpldControlShadowReg |= PLB_INVERT_mask;
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = bCpldControlShadowReg;
                            myPwm.Write(bSetCpldPwmControl, 0, bSetCpldPwmControl.Length);

                            // Clear control reg command array byte
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = 0;
                        }
                        else
                        {
                            byte bClearMask = PLB_INVERT_mask;
                            bCpldControlShadowReg &= (byte) ~bClearMask;
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = bCpldControlShadowReg;
                            myPwm.Write(bSetCpldPwmControl, 0, bSetCpldPwmControl.Length);

                            // Clear control reg command array byte
                            bSetCpldPwmControl[bSetCpldPwmControl.Length - 1] = 0;
                        }

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    default:
                    {
                        // return false
                        break;
                    }
                }
            }

            return (eReturnVal);
        }

        public PWM_Status PWM_SetDutyCycle(PWM_Channel ePWMChannel, int iDutyCycPerCent, SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;

            // Set invert state
            if (myPwm != null)
            {
                switch (ePWMChannel)
                {
                    case PWM_Channel.ePWM1:
                    {
                        byte[] bReadBuf = new byte[2];
                        int iTerminalCount;

                        // Read back terminal count
                        myPwm.Write(bSetReadTerminalCount, 0, bSetReadTerminalCount.Length);
//->                        myPwm.Read(bReadBuf, 0, 2, 1);
                        myPwm.Read(bReadBuf, 0, 2);
                        iTerminalCount = ((int)bReadBuf[0] << 8) | ((int)bReadBuf[1] & 0xff);

                        int iPWMxDuty = iDutyCycPerCent * (iTerminalCount + 1);
                        iPWMxDuty /= 100;
                        iPWMxDuty -= 1;

                        bSetPWM1DutyCycle[bSetPWM1DutyCycle.Length - 2] = (byte)((iPWMxDuty >> 8) & 0xff);
                        bSetPWM1DutyCycle[bSetPWM1DutyCycle.Length - 1] = (byte)(iPWMxDuty & 0xff);

                        // Write duty cycle register
                        myPwm.Write(bSetPWM1DutyCycle, 0, bSetPWM1DutyCycle.Length);

                        // Clear duty cycle setting
                        bSetPWM1DutyCycle[bSetPWM1DutyCycle.Length - 2] = 0x00;
                        bSetPWM1DutyCycle[bSetPWM1DutyCycle.Length - 1] = 0x00;

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    case PWM_Channel.ePWM2_CPLD:
                    {
                        byte[] bReadBuf = new byte[1];
                        int iPwmD = iDutyCycPerCent * CPLD_PWM_MAX_COUNT;
                        iPwmD /= 100;
                        bSetPWM2DutyCycle[bSetPWM2DutyCycle.Length - 1] = (byte)(iPwmD & 0xff);

                        myPwm.Write(bSetPWM2DutyCycle, 0, bSetPWM2DutyCycle.Length);

                        // Clear duty cycle register
                        bSetPWM2DutyCycle[bSetPWM2DutyCycle.Length - 1] = 0x00;

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    case PWM_Channel.ePWM3_CPLD:
                    {
                        byte[] bReadBuf = new byte[1];
                        int iPwmD = iDutyCycPerCent * CPLD_PWM_MAX_COUNT;
                        iPwmD /= 100;
                        bSetPWM3DutyCycle[bSetPWM3DutyCycle.Length - 1] = (byte)(iPwmD & 0xff);

                        myPwm.Write(bSetPWM3DutyCycle, 0, bSetPWM3DutyCycle.Length);

                        // Clear duty cycle register
                        bSetPWM3DutyCycle[bSetPWM3DutyCycle.Length - 1] = 0x00;

                        // Read and return status
                        myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                        myPwm.Read(bReadBuf, 0, 1, 1);
                        myPwm.Read(bReadBuf, 0, 1);

                        eReturnVal = (PWM_Status)bReadBuf[0];
                        break;
                    }
                    default:
                    {
                        // return false
                        break;
                    }
                }
            }

            return (eReturnVal);
        }

        public PWM_Status PWM_ReadStatus(SerialPort myPwm)
        {
            PWM_Status eReturnVal = PWM_Status.eStatusPWMComHandleError;
            byte[] bReadBuf = new byte[1];

            if (myPwm != null)
            {
                myPwm.Write(bSetStatusRead, 0, bSetStatusRead.Length);
//->                myPwm.Read(bReadBuf, 0, 1, 1);
                myPwm.Read(bReadBuf, 0, 1);

                eReturnVal = (PWM_Status)bReadBuf[0];
            }

            return (eReturnVal);
        }

        public UInt16 PWM_ReadTerminalCount(SerialPort myPwm)
        {
            UInt16 uiReturnVal = 0xffff;
            int iReturnVal;
            byte[] bReadBuf = new byte[2];

            if (myPwm != null)
            {
                myPwm.Write(bSetReadTerminalCount, 0, bSetReadTerminalCount.Length);
//->                myPwm.Read(bReadBuf, 0, 2, 1);
                myPwm.Read(bReadBuf, 0, 1);
                iReturnVal = (bReadBuf[0] << 8) | bReadBuf[1];
                uiReturnVal = (UInt16) iReturnVal;
            }

            return (uiReturnVal);
        }

        public UInt16 PWM_ReadDutyCyle(SerialPort myPwm)
        {
            UInt16 uiReturnVal = 0xffff;
            int iReturnVal;
            byte[] bReadBuf = new byte[2];

            if (myPwm != null)
            {
                myPwm.Write(bSetReadDutyCycle, 0, bSetReadDutyCycle.Length);
//->                myPwm.Read(bReadBuf, 0, 2, 1);
                myPwm.Read(bReadBuf, 0, 1);
                iReturnVal = (bReadBuf[0] << 8) | bReadBuf[1];
                uiReturnVal = (UInt16)iReturnVal;
            }

            return (uiReturnVal);
        }

    }
}
